@@ -14,6 +14,65 @@ import quisk_utils
 from quisk_hardware_model import Hardware as BaseHardware
 
 DEBUG = 0
+DEBUG_I2C = 0
+
+class IOBoard:
+  "This class controls the N2ADR IO Board for the HermesLite 2"
+  def __init__(self, hardware):
+    self.DEBUG = 0
+    self.hardware = hardware
+    self.have_IO_Board = None
+    self.have_board_counter = 3
+    self.tx_timer = 0
+    self.current_tx_freq = 0
+  def HeartBeat(self):	# Called at 10 Hz for housekeeping tasks
+    if not QS.get_params('rx_udp_started'):
+      return
+    if self.have_IO_Board is None:
+      resp = self.hardware.ReadI2C(0x7d, 0x41, 0)	# Check for the N2ADR HL2 IO Board
+      if resp and resp[1] == 0xF1:
+        self.have_IO_Board = True
+        if self.DEBUG or DEBUG_I2C:
+          print ('Have IO_Board')
+        self.hardware.ImmediateChange('hermes_iob_rxin')
+      else:
+        self.have_board_counter -= 1
+        if self.have_board_counter == 0:
+          self.have_IO_Board = False
+          if self.DEBUG or DEBUG_I2C:
+            print ('No IO board')
+    if not self.have_IO_Board:
+      return
+    if self.hardware.tx_frequency != self.current_tx_freq and time.time() - self.tx_timer > 0.50:
+      self.current_tx_freq = self.hardware.tx_frequency
+      self.tx_timer = time.time()
+      tx = self.current_tx_freq		# Send Tx frequency to IO Board
+      self.hardware.WriteI2C(0x7d, 0x1D, 0, (tx >> 32) & 0xFF)	# MSB
+      self.hardware.WriteI2C(0x7d, 0x1D, 1, (tx >> 24) & 0xFF)
+      self.hardware.WriteI2C(0x7d, 0x1D, 2, (tx >> 16) & 0xFF)
+      self.hardware.WriteI2C(0x7d, 0x1D, 3, (tx >>  8) & 0xFF)
+      self.hardware.WriteI2C(0x7d, 0x1D, 13, tx        & 0xFF)	# LSB
+      #self.Receive()
+  def Receive(self, register=0):	# Get the response from the IO board
+    if not self.have_IO_Board:
+      return
+    ret = self.hardware.ReadI2C(0x7d, 0x1D, register)
+    if self.DEBUG:
+      print ('IO Board: 0x%X 0x%X 0x%X 0x%X 0x%X' % tuple(ret))
+    return ret
+  def FanLevel(self, level):
+    if not self.have_IO_Board:
+      return
+    # Send the fan level as I2C register 2
+    # level is an integer from 0 to 255
+    self.hardware.WriteI2C(0x7d, 0x1D, 12, level)
+  def AuxRxInput(self, mode):
+    # 0: The HL2 operates as usual. The receive input is not used. The Pure Signal input is available.
+    # 1: The receive input is used instead of the usual HL2 Rx input. Pure Signal is not available.
+    # 2: The receive input is used for Rx, and the Pure Signal input is used during Tx.
+    if not self.have_IO_Board:
+      return
+    self.hardware.WriteI2C(0x7d, 0x1D, 11, mode)
 
 class Hardware(BaseHardware):
   var_rates = ['48', '96', '192', '384']
@@ -38,9 +97,10 @@ class Hardware(BaseHardware):
     self.repeater_freq = None		# original repeater output frequency
     self.antenna_labels = ('Ant 0', 'Ant 1')	# labels for the antenna button
     self.antenna_index = 0			# index of antenna in use
-    self.delay_config = 30			# Delay sending message to HL2 until after sound starts
+    self.delay_config = True			# Delay sending message to HL2 until after sound starts
     self.TFRC_counter = 0		# Call for power etc. at intervals
     self.key_was_down = 0
+    self.io_board = IOBoard(self)
     try:
       self.repeater_delay = conf.repeater_delay		# delay for changing repeater frequency in seconds
     except:
@@ -67,6 +127,15 @@ class Hardware(BaseHardware):
     #    C0 is a six-bit address. If the address is OR'd with 0x40, an ACK must be received or the item is re-transmitted.
     #    The item sent to the HL2 is C0<<1 | MoxBit.
     #    Sound must be started before you can use pc2hermeswritequeue.
+    # General I2C bus Access:
+    # Address C0 = 0x3c writes to the I2C bus 1. Use C0 = 0x7c to request an ACK.
+    # Address C0 = 0x3d writes to the I2C bus 2. Use C0 = 0x7d to request an ACK.
+    # Read requests must use the ACK.
+    #    self.pc2hermeslitewritequeue[0:5] = C0, C1, C2, C3, C4
+    #    C1 = 0x06 to write, 0x07 to read
+    #    C2 = 7-bit I2C address | stop;  Where "stop" is 0x80 to stop at end; else zero for continue
+    #    C3 = 8-bit I2C control (often a register address)
+    #    C4 = 8-bit I2C data for write, else 0
     self.pc2hermeslitewritequeue = bytearray(4 * 5)	# Four of (C0, C1, C2, C3, C4)
     # Initialize some data
     self.pc2hermes[3] = 0x04	# C0 index == 0, C4[5:3]: number of receivers 0b000 -> one receiver; C4[2] duplex on
@@ -189,7 +258,7 @@ class Hardware(BaseHardware):
     self.config_text = st
     self.ChangeLNA(2)	# Initialize the LNA using the correct LNA code from the FPGA code version
   def open(self):
-    self.delay_config = 30			# Delay sending message to HL2 until after sound starts
+    self.delay_config = True	# Delay sending message to HL2 until after sound starts
     for name in ('hermes_tx_buffer_latency', 'keyupDelay',
         'hermes_lowpwr_tr_enable', 'hermes_PWM', 'hermes_disable_sync', 'hermes_power_amp', 'Hermes_BandDictEnTx'):
       self.ImmediateChange(name)
@@ -309,6 +378,7 @@ class Hardware(BaseHardware):
   def ChangeFrequency(self, tx_freq, vfo_freq, source='', band='', event=None):
     if tx_freq and tx_freq > 0:
       self.tx_frequency = tx_freq
+      self.io_board.HeartBeat()
       tx = int(tx_freq - self.transverter_offset)
       self.pc2hermes[ 4] = tx >> 24 & 0xff		# C0 index == 1, C1, C2, C3, C4: Tx freq, MSB in C1
       self.pc2hermes[ 5] = tx >> 16 & 0xff
@@ -353,11 +423,11 @@ class Hardware(BaseHardware):
       self.hermes_temperature, self.hermes_fwd_power, self.hermes_rev_power, self.hermes_pa_current, self.hermes_fwd_peak, self.hermes_rev_peak = QS.get_hermes_TFRC()
       if self.application.bottom_widgets:
         self.application.bottom_widgets.UpdateText()
-    if self.delay_config > 0:
-      self.delay_config -= 1
-      if self.delay_config == 0:	# Delay sending these config parameters
-        for name in ('hermes_disable_watchdog', 'hermes_reset_on_disconnect'):
-          self.ImmediateChange(name)
+    if self.delay_config and QS.get_params('rx_udp_started'):
+      self.delay_config = False
+      for name in ('hermes_disable_watchdog', 'hermes_reset_on_disconnect', 'hermes_iob_rxin'):
+        self.ImmediateChange(name)
+    self.io_board.HeartBeat()
   def RepeaterOffset(self, offset=None):	# Change frequency for repeater offset during Tx
     if offset is None:		# Return True if frequency change is complete
       if time.time() > self.repeater_time0 + self.repeater_delay:
@@ -486,6 +556,10 @@ class Hardware(BaseHardware):
     if DEBUG: print ("Change LNA to", value)
   def ChangeTxLNA(self, value):		# LNA for Tx
     # value is -12 to +48
+    if value < -12:
+      value = -12
+    elif value > 48:
+      value = 48
     value = ((value+12) & 0x3f) | 0x40 | 0x80
     self.SetControlByte(0x0e, 3, value, False)		# C0 index == 0x0e, C3
     QS.pc_to_hermes(self.pc2hermes)
@@ -638,6 +712,14 @@ class Hardware(BaseHardware):
       self.ChangeBandFilters()
     elif name == 'hermes_antenna_tuner':
       pass
+    elif name == 'hermes_iob_rxin':
+      mode = self.conf.hermes_iob_rxin[0:4]
+      if mode == 'J10 ':
+        self.io_board.AuxRxInput(0)
+      elif mode == 'HL2 ':
+        self.io_board.AuxRxInput(1)
+      elif mode == 'Use ':
+        self.io_board.AuxRxInput(2)
     else:
       if DEBUG: print ("Immediate change: no such name", name)
   def EnableBiasChange(self, enable):
@@ -702,6 +784,8 @@ class Hardware(BaseHardware):
       if dt > 0.300:
         print("ERROR: Hermes-Lite write queue timeout, queue 0x%x 0x%x 0x%x 0x%x 0x%x" % tuple(self.pc2hermeslitewritequeue[0:5]))
         return
+    if qlen <= 0:	# Do not add to queue, just finish last write
+      return
     ## Send next write(s)
     QS.pc_to_hermeslite_writequeue(self.pc2hermeslitewritequeue)
     QS.set_hermeslite_writepointer(qlen)
@@ -865,6 +949,49 @@ class Hardware(BaseHardware):
       else:
         if DEBUG: print("EEPROM read {0:#x} from address {1:#x}".format(v0,addr))
       return v0
+  def WriteI2C(self, bus, i2caddr, control, value):
+    # bus is 0x7c or 0x7d
+    self.WriteQueue(0)		# Finish any writes
+    self.pc2hermeslitewritequeue[0:5] = bus, 0x06, i2caddr, control, value
+    QS.clear_hermeslite_response()
+    self.WriteQueue(1)
+    if bus & 0x40:	# response was requested
+      for j in range(50):
+        time.sleep(0.001)
+        resp = QS.get_hermeslite_response()
+        if resp[0] != 0:
+          #print ("0x%X 0x%X 0x%X 0x%X 0x%X" % tuple(resp))
+          if bus & 0x3F == (resp[0] >> 1) & 0x3F:
+            if DEBUG_I2C or DEBUG:
+              print ("Write I2C bus OK 0x%X, i2caddr 0x%X, control 0x%X, value 0x%X" % (bus, i2caddr, control, value))
+          else:
+            if DEBUG_I2C or DEBUG:
+              print ("Write I2C bus ERROR 0x%X, i2caddr 0x%X, control 0x%X, value 0x%X" % (bus, i2caddr, control, value))
+          break
+      else:
+        if DEBUG_I2C or DEBUG:
+          print ("Write I2C bus TIMEOUT 0x%X, i2caddr 0x%X, control 0x%X, value 0x%X" % (bus, i2caddr, control, value))
+    else:
+      if True or DEBUG: print ("Write I2C bus 0x%X, i2caddr 0x%X, control 0x%X, value 0x%X" % (bus, i2caddr, control, value))
+  def ReadI2C(self, bus, i2caddr, control):
+    # bus is 0x7c or 0x7d
+    self.WriteQueue(0)		# Finish any writes
+    self.pc2hermeslitewritequeue[0:5] = bus, 0x07, i2caddr, control, 0
+    QS.clear_hermeslite_response()
+    self.WriteQueue(1)
+    for j in range(50):
+      time.sleep(0.001)
+      resp = QS.get_hermeslite_response()
+      if resp[0] != 0:
+        break
+    else:
+      if DEBUG_I2C or DEBUG:
+        print("ReadI2C timed out and did not return a value")
+      return None
+    resp[0] = (resp[0] >> 1) & 0x3F	# 6-bit bus in C0
+    if DEBUG_I2C or DEBUG:
+      print ("Read  I2C bus 0x%X, 0x%X, 0x%X, 0x%X, 0x%X " % tuple(resp))
+    return resp
   def ProgramGateware(self, event):	# Program the Gateware (FPGA firmware) over Ethernet
     title = "Program the Gateware"
     main_frame = self.application.main_frame
