@@ -27,6 +27,8 @@ class BottomWidgets:	# Add extra widgets to the bottom of the screen
     bw, bh = b.GetMinSize()
     init = self.config.hermes_LNA_dB
     self.sliderLNA = app.SliderBoxHH(frame, 'RfLna %d dB', init, -12, 48, self.OnLNA, True)
+    self.sliderLNA.idName = "RfLna"
+    app.midiControls["RfLna"]	= (self.sliderLNA,	self.OnLNA)
     hardware.ChangeLNA(init)
     gbs.Add(self.sliderLNA, (start_row, self.start_col + 2), (1, 8), flag=wx.EXPAND)
     if conf.button_layout == "Small screen":
