@@ -60,6 +60,7 @@ struct alc {
 int mic_max_display;				// display value of maximum microphone signal level 0 to 2**15 - 1
 int quiskSpotLevel = -1;			// level is -1 for Spot button Off; else the Spot level 0 to 1000.
 int quiskImdLevel = 500;			// level for rxMode IMD, 0 to 1000
+int hermes_mox_bit;				// mox bit to send to Hermes
 
 static SOCKET mic_socket = INVALID_SOCKET;	// send microphone samples to a socket
 static double mic_agc_level = 0.10;		// Mic levels below this are noise and are ignored
@@ -889,7 +890,7 @@ static void quisk_hermes_tx_add(complex double * cSamples, int tx_count, int key
 void quisk_hermes_tx_send(int tx_socket, int * tx_records)
 {	// Send one UDP block of mic samples using the Metis-Hermes protocol.  Timing is from blocks received, rate is 48k.
 	// If the key is up we send samples anyway, but the samples are zero.
-	int i, j, offset, sent, ratio, mox_bit;
+	int i, j, offset, sent, ratio;
 	short s;
 	unsigned char sendbuf[1032];
 	unsigned char * pt_buf;
@@ -909,15 +910,6 @@ void quisk_hermes_tx_send(int tx_socket, int * tx_records)
 		last_play_state = quisk_play_state;
 		serial_key_samples(NULL, 0);
 	}
-	if (quisk_is_key_down()) {
-		if (quisk_play_state == HARDWARE_CWKEY)
-			mox_bit = 0;
-		else
-			mox_bit = 1;
-	}
-	else {
-		mox_bit = 0;
-	}
 	//printf("hermes_tx_send start 2: hermes_num_samples %d\n", hermes_num_samples);
 	ratio = quisk_sound_state.sample_rate / 48000;		// send rate is 48 ksps
 	//printf ("quisk_hermes_tx_send ratio %d count %d\n", ratio, *tx_records);
@@ -956,14 +948,14 @@ void quisk_hermes_tx_send(int tx_socket, int * tx_records)
 	sendbuf[9] = 0x7F;
 	sendbuf[10] = 0x7F;
 	offset = C0_index * 4;		// offset into quisk_pc_to_hermes is C0[7:1] * 4
-	sendbuf[11] = C0_index << 1 | mox_bit;			// C0
+	sendbuf[11] = C0_index << 1 | hermes_mox_bit;			// C0
 	sendbuf[12] = quisk_pc_to_hermes[offset++];		// C1
 	sendbuf[13] = quisk_pc_to_hermes[offset++];		// C2
 	sendbuf[14] = quisk_pc_to_hermes[offset++];		// C3
 	sendbuf[15] = quisk_pc_to_hermes[offset++];		// C4
 	if (C0_index == 0) {	// Do not change receiver count without stopping Hermes and restarting
 		sendbuf[15] = quisk_multirx_count << 3 | 0x04;	// Send the old count, not the changed count
-		if (mox_bit)		// send filter selection on J16
+		if (hermes_mox_bit)		// send filter selection on J16
 			sendbuf[13] = hermes_filter_tx << 1;
 		else
 			sendbuf[13] = hermes_filter_rx << 1;
@@ -996,7 +988,7 @@ void quisk_hermes_tx_send(int tx_socket, int * tx_records)
 
 		// Only send periodic hermeslite writes in second part of frame
 		hlwp = 5*(quisk_hermeslite_writepointer-1);
-		sendbuf[523] = quisk_hermeslite_writequeue[hlwp++] << 1 | mox_bit;
+		sendbuf[523] = quisk_hermeslite_writequeue[hlwp++] << 1 | hermes_mox_bit;
 		sendbuf[524] = quisk_hermeslite_writequeue[hlwp++];
 		sendbuf[525] = quisk_hermeslite_writequeue[hlwp++];
 		sendbuf[526] = quisk_hermeslite_writequeue[hlwp++];
@@ -1009,14 +1001,14 @@ void quisk_hermes_tx_send(int tx_socket, int * tx_records)
 		}
 	} else {
 		offset = C0_index * 4;		// offset into quisk_pc_to_hermes is C0[7:1] * 4
-		sendbuf[523] = C0_index << 1 | mox_bit;		// C0
+		sendbuf[523] = C0_index << 1 | hermes_mox_bit;		// C0
 		sendbuf[524] = quisk_pc_to_hermes[offset++];		// C1
 		sendbuf[525] = quisk_pc_to_hermes[offset++];		// C2
 		sendbuf[526] = quisk_pc_to_hermes[offset++];		// C3
 		sendbuf[527] = quisk_pc_to_hermes[offset++];		// C4
 		if (C0_index == 0) {
 			sendbuf[527] = quisk_multirx_count << 3 | 0x04;		// Send the old count, not the changed count
-			if (mox_bit)		// send filter selection on J16
+			if (hermes_mox_bit)		// send filter selection on J16
 				sendbuf[525] = hermes_filter_tx << 1;
 			else
 				sendbuf[525] = hermes_filter_rx << 1;
@@ -1259,7 +1251,7 @@ int quisk_process_microphone(int mic_sample_rate, complex double * cSamples, int
 	}
 	if (key_down != key_was_down) {	
 		key_down_counter = quisk_start_ssb_delay * 48;	// Key was pressed. Zero the first few samples to clear the buffers.
-		init_alc(&tx_alc, 0);		// init ALC for Tx and RECORD_MIC
+		init_alc(&tx_alc, 0);		// init ALC for Tx and TMP_RECORD_MIC
 		key_was_down = key_down;
 	}
 	if (quisk_play_state != last_play_state) {
@@ -1277,21 +1269,21 @@ int quisk_process_microphone(int mic_sample_rate, complex double * cSamples, int
 		else switch (rxMode) {
 		case LSB:		// LSB
 		case USB:		// USB
-			if (quisk_record_state == PLAYBACK)
+			if (quisk_record_state == TMP_PLAY_SPKR_MIC)
 				count = tx_filter_digital(cSamples, count);	// filter samples, minimal processing
 			else
 				count = tx_filter(cSamples, count);		// filter samples
 			process_alc(cSamples, count, &tx_alc, rxMode);
 			break;
 		case AM:		// AM
-			if (quisk_record_state != PLAYBACK)		// no audio processing for recorded sound
+			if (quisk_record_state != TMP_PLAY_SPKR_MIC)		// no audio processing for recorded sound
 				count = tx_filter(cSamples, count);
 			for (i = 0; i < count; i++)	// transmit (0.5 + ampl/2, 0)
 				cSamples[i] = (creal(cSamples[i]) + CLIP16) * 0.5;
 			process_alc(cSamples, count, &tx_alc, rxMode);
 			break;
 		case FM:		// FM
-			if (quisk_record_state != PLAYBACK)		// no audio processing for recorded sound
+			if (quisk_record_state != TMP_PLAY_SPKR_MIC)		// no audio processing for recorded sound
 				count = tx_filter(cSamples, count);
 			if (quisk_ctcss_freq > 9) {
 				ctcss_delta = 2.0 * M_PI / MIC_OUT_RATE * quisk_ctcss_freq;
@@ -1402,7 +1394,7 @@ int quisk_process_microphone(int mic_sample_rate, complex double * cSamples, int
 	else if (quisk_use_rx_udp && key_down) {	// Send mic samples to UDP when key is down
 		transmit_udp(cSamples, count);
 	}
-	if (quisk_record_state == RECORD_MIC) {
+	if (quisk_record_state == TMP_RECORD_MIC) {
 		switch (rxMode) {
 		case LSB:		// LSB
 		case USB:		// USB
