@@ -27,7 +27,7 @@
 // 3 == Copy transmit audio to the output.
 #define TEST_AUDIO	0
 
-#define QUISK_CWKEY_DOWN	(quisk_hardware_cwkey || quisk_serial_key_down || quisk_midi_cwkey)
+#define QUISK_CWKEY_DOWN	(quisk_hardware_cwkey || quisk_serial_key_down || quisk_midi_cwkey || quisk_remote_cwkey)
 
 #ifdef MS_WINDOWS
 #define QUISK_SHUT_RD	SD_RECEIVE
@@ -198,7 +198,7 @@ struct sound_conf {
 	int overrange;		// count of ADC overrange (clip) for non-soundcard device
 	int latencyCapt;
 	int latencyPlay;
-	int interupts;
+	int interrupts;
 	char msg1[QUISK_SC_SIZE];
 	char err_msg[QUISK_SC_SIZE];
 	// These parameters are for the microphone:
@@ -220,17 +220,16 @@ struct sound_conf {
 
 enum quisk_rec_state {
 	IDLE,
-	RECORD_RADIO,
-	RECORD_MIC,
-	PLAYBACK,
-	PLAY_FILE,
-	PLAY_SAMPLES } ;
+	TMP_RECORD_SPEAKERS,
+	TMP_RECORD_MIC,
+	TMP_PLAY_SPKR_MIC,
+	FILE_PLAY_SPKR_MIC,
+	FILE_PLAY_SAMPLES } ;
 extern enum quisk_rec_state quisk_record_state;
 
 struct wav_file {
 	FILE * fp;
 	char file_name[QUISK_PATH_SIZE];
-	int enable;
 	unsigned long samples;
 };
 
@@ -247,6 +246,20 @@ struct QuiskWav {			// data to create a WAV or RAW audio file
     int fpPos;
 } ;
 
+// Remote Quisk control head and slave by Ben, AC2YD
+extern int remote_control_head;
+extern int remote_control_slave;
+int read_remote_radio_sound_socket(struct sound_dev* dev, complex double * cSamples, int nSamples);
+int read_remote_mic_sound_socket(struct sound_dev* dev, complex double * cSamples);
+void send_remote_radio_sound_socket(complex double * cSamples, int nSamples);
+void send_remote_mic_sound_socket(complex double * cSamples, int nSamples);
+extern PyObject * quisk_start_control_head_remote_sound(PyObject * self, PyObject * args);
+extern PyObject * quisk_stop_control_head_remote_sound(PyObject * self, PyObject * args);
+extern PyObject * quisk_start_remote_radio_remote_sound(PyObject * self, PyObject * args);
+extern PyObject * quisk_stop_remote_radio_remote_sound(PyObject * self, PyObject * args);
+extern int receive_graph_data(double * fft_avg);
+extern void send_graph_data(double * fft_avg, int fft_size, double zoom, double deltaf, int fft_sample_rate, double scale);
+
 void QuiskWavClose(struct QuiskWav *);
 int QuiskWavWriteOpen(struct QuiskWav *, char *, short, short, short, int, double);
 void QuiskWavWriteC(struct QuiskWav *, complex double *, int);
@@ -257,6 +270,7 @@ void QuiskWavReadC(struct QuiskWav *, complex double *, int);
 void QuiskWavReadD(struct QuiskWav *, double *, int);
 void QuiskMeasureRate(const char *, int, int, int);
 void quisk_record_audio(struct wav_file *, complex double *, int);
+void copy2pixels(double * pixels, int n_pixels, double * fft, int fft_size, double zoom, double deltaf, double rate);
 
 extern struct sound_conf quisk_sound_state, * pt_quisk_sound_state;
 extern int mic_max_display;		// display value of maximum microphone signal level
@@ -281,6 +295,7 @@ extern unsigned int quisk_hermeslite_writepointer;		// write pointer into write
 extern unsigned int quisk_hermeslite_writeattempts;		// counter for write retries
 extern unsigned int quisk_hermes_code_version;			// Hermes code version from Hermes to PC
 extern unsigned int quisk_hermes_board_id;			// Hermes board ID from Hermes to PC
+extern int hermes_mox_bit;					// Hermes mox bit from the PC to Hermes
 extern int quisk_use_rx_udp;					// Method of access to UDP hardware
 extern complex double cRxFilterOut(complex double, int, int);
 extern int quisk_multirx_count;			// number of additional receivers zero or 1, 2, 3, ..
@@ -292,6 +307,7 @@ extern int quisk_serial_key_down;		// The state of the serial port CW key
 extern int quisk_serial_ptt;			// The state of the serial port PTT
 extern int quisk_hardware_cwkey;		// The state of the hardware CW key from UDP or USB
 extern int quisk_midi_cwkey;			// The state of the MIDI CW key
+extern int quisk_remote_cwkey;			// The state of the remote CW key
 extern int quisk_sidetoneFreq;			// Frequency in hertz for the sidetone
 extern int quisk_active_sidetone;		// Whether and how to generate a sidetone
 extern int quisk_isFDX;				// Are we in full duplex mode?
@@ -303,6 +319,7 @@ extern int n_speech_sample_rate;		// Microphone decimate to speech_sample_rate,
 extern int quisk_start_cw_delay;		// milliseconds to delay output on serial or MIDI CW key down
 extern int quisk_start_ssb_delay;		// milliseconds to discard output for all modes except CW
 extern struct sound_dev * quiskPlaybackDevices[];	// array of Playback sound devices
+extern int quisk_close_file_play;
 
 extern PyObject * quisk_set_spot_level(PyObject * , PyObject *);
 extern PyObject * quisk_get_tx_filter(PyObject * , PyObject *);
@@ -380,7 +397,7 @@ void quisk_hermes_tx_send(int, int *);
 void quisk_udp_mic_error(char *);
 void quisk_check_freedv_mode(void);
 void quisk_calc_audio_graph(double, complex double *, double *, int, int);
-int QuiskDeltaMsec(int);
+double QuiskDeltaSec(int);
 void * quisk_make_sidetone(struct sound_dev *, int);
 void * quisk_make_txIQ(struct sound_dev *, int);
 int quisk_play_sidetone(struct sound_dev *);
