@@ -408,7 +408,7 @@ Hware_Hl2_EepromMACUse = 'Ignore'
 ## hermes_TxLNA_dB			LNA during Tx dB, integer
 # During transmit the low noise Rx amplifier gain changes to this value (in dB) if the hardware supports it.
 # Changes are immediate (no need to restart).
-hermes_TxLNA_dB = 21
+hermes_TxLNA_dB = -12
 
 ## hermes_tx_buffer_latency		Tx buffer msec, integer
 # This is the latency of the transmit buffer in milliseconds.
@@ -464,6 +464,17 @@ hermes_reset_on_disconnect = False
 hermes_bias_adjust = False
 #hermes_bias_adjust = True
 
+## hermes_iob_rxin			IO board Rx input, text choice
+# This controls the usage of the Rx input J9 and the Pure Signal input J10 on the N2ADR IO board. This option
+# has no effect if the IO board is not installed.
+# The first option disables the Rx input at J9 and the HL2 operates as usual. The Pure Signal input at J10 is available.
+# The second option connects the HL2 receive input to J9. J10 is not available.
+# The third option connects the HL2 receive input to J9 on Rx and to J10 on Tx.
+# Changes are immediate (no need to restart).
+hermes_iob_rxin = 'J10 available'
+#hermes_iob_rxin = 'HL2 Rx to J9'
+#hermes_iob_rxin = 'Use J9 and J10'
+
 
 # These are known power meter calibration tables. This table is not present in the JSON settings file.
 power_meter_std_calibrations = {}
@@ -851,6 +862,19 @@ Hermes_BandDictEnTx = False
 #default_rf_gain = 11
 
 
+################ Receivers Control Head, Use Quisk to control a remote radio. No real hardware is used here.
+## hardware_file_name		Hardware file path, rfile
+# Quisk can be used as a control head to control a real radio located remotely.
+# This file contains the control logic for each remote radio.
+hardware_file_name = 'ac2yd/control_softrock.py'
+#hardware_file_name = 'ac2yd/control_hermes.py'
+
+## widgets_file_name		Widget file path, rfile
+# This file adds additional controls for the radio. Choose the correct file (if any) for the remote radio.
+widgets_file_name = ''
+#widgets_file_name = 'hermes/quisk_widgets.py'
+
+
 ################ Sound
 # Playback devices:
 #    name_of_sound_play      Play radio sound on speakers or headphones
@@ -1295,7 +1319,7 @@ dxClPassword = ''
 dxClExpireTime = 20
 
 ## IQ_Server_IP         Pulse server IP address, text
-#IP Adddress for remote PulseAudio IQ server.
+#IP Address for remote PulseAudio IQ server.
 IQ_Server_IP = ""
 
 ## hamlib_ip            IP address for Hamlib Rig 2, text
@@ -1351,6 +1375,20 @@ win_hamlib_com2_name = ""
 #win_hamlib_com2_name = "COM15"
 #win_hamlib_com2_name = "COM16"
 
+## remote_radio_ip         Remote radio IP or name, text
+# Quisk can be used as a control head to control a real radio located remotely.
+# This is the IP adddress or the host name of the remote radio.
+remote_radio_ip = ""
+#remote_radio_ip = "192.168.1.56"
+
+## remote_radio_password         Password for remote radio, text
+# Quisk can be used as a control head to control a real radio located remotely.
+# For security it requires the same password to be entered on both computers.
+# Enter a fairly long pass phrase of 20 or more characters.
+# It is only necessary to enter it once on each computer.
+remote_radio_password = ""
+
+
 
 hamlib_com1_name = ""
 hamlib_com2_name = ""
@@ -1410,6 +1448,13 @@ midi_cwkey_note = -1
 #midi_cwkey_note = 60
 #midi_cwkey_note = 69
 
+## midi_ptt_toggle	Midi PTT toggle, boolean
+# Set to True if you want PTT to remain on when you release the Midi PTT button.  A second button press will
+# then release PTT.  This is toggle mode.  If False, you must keep pressing the button, and releasing
+# it will release PTT.
+# Changes are immediate (no need to restart).
+midi_ptt_toggle = False
+#midi_ptt_toggle = True
 
 
 
@@ -1623,7 +1668,7 @@ cwTone = 600
 
 ## use_sidetone				Use sidetone, integer choice
 # This controls whether Quisk will display a sidetone volume control
-# and gererate a CW sidetone.
+# and generate a CW sidetone.
 use_sidetone = 0
 #use_sidetone = 1
 
