@@ -4,7 +4,7 @@ from __future__ import print_function
 from __future__ import absolute_import
 from __future__ import division
 
-import sys, re
+import sys, re, json
 import wx, wx.lib.buttons, wx.lib.stattext
 # The main script will alter quisk_conf_defaults to include the user's config file.
 import quisk_conf_defaults as conf
@@ -227,6 +227,7 @@ class SliderBoxH:
     self.handler = handler
     self.display = display
     self.scale = scale
+    self.idName = text
     if display:		# Display the slider value
       t1 = self.text % (themin * scale)
       t2 = self.text % (themax * scale)
@@ -260,6 +261,8 @@ class SliderBoxH:
     if event:
       event.Skip()
       if self.handler:
+        if application.remote_control_head:
+          application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetValue()}\n')
         self.handler(event)
     if self.display:
       t = self.text % (self.slider.GetValue() * self.scale)
@@ -283,13 +286,15 @@ class SliderBoxH:
     value = int(value + 0.5)
     self.SetValue(value)
     if do_cmd:
+      if application.remote_control_head:
+        application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetValue()}\n')
       self.handler(None)
 
 class SliderBoxHH(SliderBoxH, wx.BoxSizer):
   """A horizontal control with a slider and text with a value.  The text must have a %d if display is True."""
-  def __init__(self, parent, text, init, themin, themax, handler, display):
+  def __init__(self, parent, text, init, themin, themax, handler, display, scale=1):
     wx.BoxSizer.__init__(self, wx.HORIZONTAL)
-    SliderBoxH.__init__(self, parent, text, init, themin, themax, handler, display, None, None)
+    SliderBoxH.__init__(self, parent, text, init, themin, themax, handler, display, None, None, scale)
     #font = wx.Font(10, wx.FONTFAMILY_SWISS, wx.NORMAL, wx.FONTWEIGHT_NORMAL, False, conf.quisk_typeface)
     #self.text_ctrl.SetFont(font)
     self.Add(self.text_ctrl, 0, wx.ALIGN_CENTER)
@@ -302,8 +307,9 @@ class SliderBoxV(wx.BoxSizer):
   def __init__(self, parent, text, init, themax, handler, display=False, themin=0):
     wx.BoxSizer.__init__(self, wx.VERTICAL)
     self.slider = wx.Slider(parent, -1, init, themin, themax, style=wx.SL_VERTICAL|wx.SL_INVERSE)
-    self.slider.Bind(wx.EVT_SCROLL, handler)
+    self.slider.Bind(wx.EVT_SCROLL, self.OnScroll)
     self.handler = handler
+    self.idName = text
     sw, sh = self.slider.GetSize()
     self.text = text
     self.themin = themin
@@ -337,6 +343,10 @@ class SliderBoxV(wx.BoxSizer):
     self.text_ctrl.SetForegroundColour(parent.GetForegroundColour())
     self.Add(self.text_ctrl, 0, 0)
     self.Add(self.slider, 1, 0)
+  def OnScroll(self, event):
+    if application.remote_control_head:
+      application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetValue()}\n')
+    self.handler(event)
   def Change(self, event):
     event.Skip()
     self.text_ctrl.SetLabel(str(self.slider.GetValue()))
@@ -359,7 +369,7 @@ class SliderBoxV(wx.BoxSizer):
     value = int(value + 0.5)
     self.SetValue(value)
     if do_cmd:
-      self.handler(None)
+      self.OnScroll(None)
 
 class QuiskText1(wx.lib.stattext.GenStaticText):
   # Self-drawn text for QuiskText.
@@ -429,6 +439,8 @@ class QuiskText(wx.BoxSizer):
     self.Add(self.TextCtrl, 1, flag=wx.ALIGN_CENTER_VERTICAL)
   def SetLabel(self, label):
     self.TextCtrl.SetLabel(label)
+  def GetLabel(self):
+    return self.TextCtrl.GetLabel()
 
 # Start of our button classes.  They are compatible with wxPython GenButton
 # buttons.  Use the usual methods for access:
@@ -573,6 +585,8 @@ class QuiskPushbutton(QuiskButtons, wx.lib.buttons.GenButton):
       self.Bind(wx.EVT_RIGHT_UP, self.OnRightUp)
   def OnButton(self, event):
     if self.command:
+      if application.remote_control_head:
+        application.Hardware.RemoteCtlSend(self.idName + ';1\n')
       self.command(event)
   def OnRightDown(self, event):
     self.direction = -1
@@ -580,6 +594,13 @@ class QuiskPushbutton(QuiskButtons, wx.lib.buttons.GenButton):
   def OnRightUp(self, event):
     self.OnLeftUp(event)
     self.direction = 1
+  def SetIndex(self, index):
+    if self.command and index:
+      if application.remote_control_head:
+        application.Hardware.RemoteCtlSend(self.idName + ';1\n')
+      event = wx.PyEvent()
+      event.SetEventObject(self)
+      self.command(event)
       
 
 class QuiskRepeatbutton(QuiskButtons, wx.lib.buttons.GenButton):
@@ -671,11 +692,22 @@ class QuiskCheckbutton(QuiskButtons, wx.lib.buttons.GenToggleButton):
     wx.lib.buttons.GenToggleButton.SetValue(self, value)
     self.button_down = value
     if do_cmd and self.command:
+      if application.remote_control_head:
+        application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetIndex()}\n')
       event = wx.PyEvent()
       event.SetEventObject(self)
       self.command(event)
+  def SetIndex(self, index, do_cmd=False):
+    self.SetValue(bool(index), do_cmd)
+  def GetIndex(self):
+    if self.GetValue():
+      return 1
+    else:
+      return 0
   def OnButton(self, event):
     if self.command:
+      if application.remote_control_head:
+        application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetIndex()}\n')
       self.command(event)
   def OnRightDown(self, event):
     self.direction = -1
@@ -896,6 +928,8 @@ class WrapControl(wx.BoxSizer):
     return self.button.GetValue()
   def SetValue(self, value, do_cmd=False):
     self.button.SetValue(value, do_cmd)
+  def SetIndex(self, index, do_cmd=False):
+    self.button.SetIndex(index, do_cmd)
   def GetLabel(self):
     return self.button.GetLabel()
   def __getattr__(self, name):
@@ -936,11 +970,12 @@ class WrapMenu(WrapControl):
     self.Add(button, 1, flag=wx.ALIGN_CENTER_VERTICAL)
     b = QuiskBitmapButton(button.GetParent(), self.OnPopButton, _bitmap_menupop)
     self.Add(b, 0, flag=wx.ALIGN_CENTER_VERTICAL|wx.LEFT, border=2)
+    self.pop_button = b
   def OnPopButton(self, event):
     if self.on_open:
       self.on_open(self.menu)
     pos = (5, 5)
-    self.button.PopupMenu(self.menu, pos)
+    self.pop_button.PopupMenu(self.menu, pos)
 
 class WrapSlider(WrapControl):
   def __init__(self, button, command, slider_value=0, slider_min=0, slider_max=1000, display=False, wintype=''):
@@ -1002,6 +1037,8 @@ class WrapSlider(WrapControl):
       self.button.SetLabel(str(value))
       self.button.Refresh()
     if self.command:
+      if application.remote_control_head:
+        application.Hardware.RemoteCtlSend(f'{self.idName}Slider;{value}\n')
       event = wx.PyEvent()
       event.SetEventObject(self.button)
       self.command(event)
@@ -1027,7 +1064,6 @@ class WrapDualSlider(WrapControl):	# Thanks to Steve, KF7O
     self.display = display                  # Display the value at the top
     WrapControl.__init__(self)
     self.Add(button, 1, flag=wx.ALIGN_CENTER_VERTICAL)
-    print("Size",self.button.GetSize())
     ## This is a hack to get _bitmap_sliderpop
     ## It would be better if _bitmap_sliderpop were not a global variable 
     ##but a first-class member in another module
@@ -1076,6 +1112,8 @@ class QuiskCycleCheckbutton(QuiskCheckbutton):
     QuiskCheckbutton.SetLabel(self, label)
     QuiskCheckbutton.SetValue(self, self.index)
     if do_cmd and self.command:
+      if application.remote_control_head:
+        application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetIndex()}\n')
       event = wx.PyEvent()
       event.SetEventObject(self)
       self.command(event)
@@ -1084,9 +1122,13 @@ class QuiskCycleCheckbutton(QuiskCheckbutton):
     QuiskCheckbutton.SetLabel(self, self.labels[index])
     QuiskCheckbutton.SetValue(self, index)
     if do_cmd and self.command:
+      if application.remote_control_head:
+        application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetIndex()}\n')
       event = wx.PyEvent()
       event.SetEventObject(self)
       self.command(event)
+  def GetIndex(self):
+    return self.index
   def OnButton(self, event):
     if not self.is_radio or self.button_down:
       self.direction = 1
@@ -1097,6 +1139,8 @@ class QuiskCycleCheckbutton(QuiskCheckbutton):
     else:
       self.direction = 0
     if self.command:
+      if application.remote_control_head:
+        application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetIndex()}\n')
       self.command(event)
   def OnRightDown(self, event):		# Move left in the list of labels
     if not self.is_radio or self.GetValue():
@@ -1106,6 +1150,8 @@ class QuiskCycleCheckbutton(QuiskCheckbutton):
       self.SetIndex(self.index)
       self.direction = -1
       if self.command:
+        if application.remote_control_head:
+          application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetIndex()}\n')
         self.command(event)
   def OnLeftDclick(self, event):	# Left double-click: Set index zero
     if not self.is_radio or self.GetValue():
@@ -1113,6 +1159,8 @@ class QuiskCycleCheckbutton(QuiskCheckbutton):
       self.SetIndex(self.index)
       self.direction = 1
       if self.command:
+        if application.remote_control_head:
+          application.Hardware.RemoteCtlSend(f'{self.idName};{self.GetIndex()}\n')
         self.command(event)
   def DrawLabel(self, dc, width, height, dx=0, dy=0):
     QuiskCheckbutton.DrawLabel(self, dc, width, height, dx, dy)
@@ -1171,6 +1219,9 @@ class RadioButtonGroup:
     else:
       button.command = self.OnButton
   def SetLabel(self, label, do_cmd=False, direction=None):
+    if application.remote_control_head:
+      func = "%s.SetLabel" % self.idName
+      application.Hardware.RemoteCtlSend(f'JsonAppFunc;{json.dumps((func, label, do_cmd, direction))}\n')
     self.button = None
     for b in self.buttons:
       if self.button is not None:
@@ -1460,3 +1511,45 @@ class FreqSetter(wx.TextCtrl):
     except:
       value = 7000
     return value
+
+class QuiskMenu(wx.Menu):
+  def __init__(self, menu_name):
+    wx.Menu.__init__(self)
+    self.menu_name = menu_name
+    self.id2data = {}
+    self.item_text2id = {}
+  def InitData(self, item, handler):
+    self.Bind(wx.EVT_MENU, self.Handler, item)
+    item_text = item.GetItemLabelText()
+    self.id2data[item.GetId()] = (item_text, handler)
+    self.item_text2id[item_text] = item.GetId()
+  def Append(self, item_text, handler):
+    item = wx.Menu.Append(self, -1, item_text)
+    self.InitData(item, handler)
+    return item
+  def AppendCheckItem(self, item_text, handler, is_checked=0):
+    item = wx.Menu.AppendCheckItem(self, -1, item_text)
+    item.Check(is_checked)
+    self.InitData(item, handler)
+    return item
+  def AppendRadioItem(self, item_text, handler, is_checked=0):
+    item = wx.Menu.AppendRadioItem(self, -1, item_text)
+    item.Check(is_checked)
+    self.InitData(item, handler)
+    return item
+  def IsItemChecked(self, item_text):
+    nid = self.item_text2id[item_text]
+    menu_item = self.FindItemById(nid)
+    return menu_item.IsChecked()
+  def Handler(self, event=None, nid=0):
+    if event:
+      nid = event.GetId()
+    item_text, handler = self.id2data[nid]
+    if application.remote_control_head:
+      menu_item = self.FindItemById(nid)
+      if menu_item.IsCheckable():
+        checked = menu_item.IsChecked()
+      else:
+        checked = 0
+      application.Hardware.RemoteCtlSend('MENU;%s;%s;%d\n' % (self.menu_name, item_text, int(checked)))
+    handler(event)
