@@ -1,19 +1,21 @@
 /*
  * Sound modules that do not depend on alsa or portaudio
 */
+#define PY_SSIZE_T_CLEAN
 #include <Python.h>
 #include <complex.h>
 #include <math.h>
 #include <sys/time.h>
 #include <time.h>
+#include <errno.h>
 
 #ifdef MS_WINDOWS
-#include <Winsock2.h>
+#include <winsock2.h>
 #include <stdint.h>
 #else
 #include <sys/socket.h>
 #include <arpa/inet.h>
-#include <netinet/in.h>
+#include <netinet/ip.h>
 #endif
 
 #include "quisk.h"
@@ -49,16 +51,22 @@ struct sound_dev quisk_Playback;
 struct sound_dev quisk_MicPlayback;
 
 static struct sound_dev Capture, MicCapture, DigitalInput, DigitalOutput, RawSamplePlayback;
-struct sound_dev quisk_DigitalRx1Output;
+static struct sound_dev dr1, dr2, dr3, dr4, dr5, dr6, dr7, dr8, dr9;		// sound output for digital sub-receivers
 // These are arrays of all capture and playback devices, and MUST end with NULL:
 static struct sound_dev * CaptureDevices[] = {&Capture, &MicCapture, &DigitalInput, NULL};
-static struct sound_dev * PlaybackDevices[] = {&quisk_Playback, &quisk_MicPlayback, &DigitalOutput, &RawSamplePlayback, &quisk_DigitalRx1Output, NULL};
+struct sound_dev * quiskPlaybackDevices[QUISK_INDEX_SUB_RX1 + QUISK_MAX_SUB_RECEIVERS + 1] = {
+	&quisk_Playback, &quisk_MicPlayback, &DigitalOutput, &RawSamplePlayback,
+	&dr1, &dr2, &dr3, &dr4, &dr5, &dr6, &dr7, &dr8, &dr9,
+	NULL};
+
 static SOCKET radio_sound_socket = INVALID_SOCKET;		// send radio sound samples to a socket
 static SOCKET radio_sound_mic_socket = INVALID_SOCKET;	// receive mic samples from a socket
 static int radio_sound_nshorts;						// number of shorts (two bytes) to send
 static int radio_sound_mic_nshorts;					// number of shorts (two bytes) to receive
 int quisk_active_sidetone;		// quisk_active_sidetone == 0 No sidetone; == 1 sidetone from wasapi; == 2 sidetone from old logic
 int quisk_midi_cwkey;			// the CW key from MIDI NoteOn/NoteOff messages
+int remote_control_head;
+int remote_control_slave;
 
 struct sound_conf quisk_sound_state;	// Current sound status
 struct sound_conf * pt_quisk_sound_state = &quisk_sound_state;
@@ -67,8 +75,7 @@ struct sound_conf * pt_quisk_sound_state = &quisk_sound_state;
 char * sound_format_names[4] = {"Int32", "Int16", "Float32", "Int24"} ;
 
 static struct wav_file file_rec_audio, file_rec_samples, file_rec_mic;
-
-static int file_record_button;			// the file record button is down
+static int close_file_rec;
 
 static double digital_output_level = 0.7;
 static int dc_remove_bw=100;			// bandwidth of DC removal filter
@@ -249,13 +256,11 @@ void quisk_record_audio(struct wav_file * wavfile, complex double * cSamples, in
 			fclose(wavfile->fp);
 		wavfile->fp = fp = fopen(wavfile->file_name, "wb");
 		if ( ! fp) {
-			wavfile->enable = 0;
 			return;
 		}
 		if (fwrite("RIFF", 1, 4, fp) != 4) {
 			fclose(fp);
 			wavfile->fp = NULL;
-			wavfile->enable = 0;
 			return;
 		}
 		// pcm data, 16-bit samples, one channel
@@ -332,13 +337,11 @@ static int record_samples(struct wav_file * wavfile, complex double * cSamples,
 			fclose(wavfile->fp);
 		wavfile->fp = fp = fopen(wavfile->file_name, "wb");
 		if ( ! fp) {
-			wavfile->enable = 0;
 			return 0;
 		}
 		if (fwrite("RIFF", 1, 4, fp) != 4) {
 			fclose(fp);
 			wavfile->fp = NULL;
-			wavfile->enable = 0;
 			return 0;
 		}
 		// IEEE float data, two channels
@@ -370,7 +373,6 @@ static int record_samples(struct wav_file * wavfile, complex double * cSamples,
 		if (wavfile->fp)
 			fclose(wavfile->fp);
 		wavfile->fp = NULL;
-		wavfile->enable = 0;
 		break;
 	default:	// write the sound data to the file
 		fp = wavfile->fp;
@@ -513,8 +515,8 @@ void play_sound_interface(struct sound_dev* dev, int nSamples, complex double *
 	static int disturb_time = 0;
 	// Damage the sample rate for debug
 	disturb_time += nSamples;
-	if (disturb_time > 8000) {
-		nSamples--;
+	if (disturb_time > 300) {
+		nSamples++;
 		disturb_time = 0;
 	}
 #endif
@@ -566,6 +568,12 @@ void play_sound_interface(struct sound_dev* dev, int nSamples, complex double *
 		if (dev->cr_average_count <= 0) {
 			dev->cr_correction = 0;
 		}
+		else if (dev->dev_index == t_MicPlayback && (rxMode == CWL || rxMode == CWU)) {
+			dev->cr_correction = 0;
+			dev->cr_average_fill /= dev->cr_average_count;
+			if (quisk_sound_state.verbose_sound > 1)
+				QuiskPrintf("%s:  Buffer average %5.2lf\n", dev->stream_description, dev->cr_average_fill * 100);
+		}
 		else if (dev->cr_delay > 0) {
 			dev->cr_delay--;
 			dev->cr_correction = 0;
@@ -824,6 +832,7 @@ int quisk_read_sound(void)	// Called from sound thread
 	double mic_play_volume;
 	complex double tx_mic_phase;
 	static double cwEnvelope=0;
+	static double cwCount=0;
 	static complex double tuneVector = (double)CLIP32 / CLIP16;	// Convert 16-bit to 32-bit samples
 	static struct quisk_cFilter filtInterp={NULL};
 	int key_state, is_DGT;
@@ -834,8 +843,14 @@ int quisk_read_sound(void)	// Called from sound thread
 	complex double tmpSamples[SAMP_BUFFER_SIZE];
 #endif
 
+	if (close_file_rec) {
+		close_file_rec = 0;
+		quisk_record_audio(&file_rec_audio, NULL, -2);
+		quisk_record_audio(&file_rec_mic, NULL, -2);
+		record_samples(&file_rec_samples, NULL, -2);
+	}
 	//QuiskPrintTime("quisk_read_sound start", 0);
-	quisk_sound_state.interupts++;
+	quisk_sound_state.interrupts++;
 	if (quisk_use_serial_port) {
 		quisk_poll_hardware_key();
 	}
@@ -900,7 +915,7 @@ int quisk_read_sound(void)	// Called from sound thread
 	}
 	else {
 		QuiskSleepMicrosec(5000);
-		nSamples = QuiskDeltaMsec(1) * quisk_sound_state.sample_rate / 1000;
+		nSamples = (int)(0.5 + QuiskDeltaSec(1) * quisk_sound_state.sample_rate);
 		if (nSamples > SAMP_BUFFER_SIZE / 2)
 			nSamples = SAMP_BUFFER_SIZE / 2;
 		for (i = 0; i < nSamples; i++)
@@ -924,7 +939,7 @@ int quisk_read_sound(void)	// Called from sound thread
 	if (RawSamplePlayback.handle)
 		play_sound_interface(&RawSamplePlayback, nSamples, cSamples, 0, 1.0);
 	// Perhaps replace the samples with samples from a file
-	if (quisk_record_state == PLAY_SAMPLES)
+	if (quisk_record_state == FILE_PLAY_SAMPLES)
 		quisk_play_samples(cSamples, nSamples);
 #if ! DEBUG_MIC
 	nSamples = quisk_process_samples(cSamples, nSamples);
@@ -933,13 +948,18 @@ int quisk_read_sound(void)	// Called from sound thread
 	snprintf(str80, 80, "  rx i/q process %d samples", nSamples);
 	QuiskPrintTime(str80, 0);
 #endif
+	// For control_head role in remote control operation, receive radio sound via UDP
+	if (remote_control_head)
+		nSamples = read_remote_radio_sound_socket(&quisk_Playback, cSamples, nSamples);
+	// For remote_radio role in remote control operation, send radio sound via UDP
+	if (remote_control_slave)
+		send_remote_radio_sound_socket(cSamples, nSamples);
 
 	is_DGT = rxMode == DGT_U || rxMode == DGT_L || rxMode == DGT_IQ || rxMode == DGT_FM;
-	if (quisk_record_state == PLAYBACK)
+	if (quisk_record_state == TMP_PLAY_SPKR_MIC)
 		quisk_tmp_playback(cSamples, nSamples, 1.0);		// replace radio sound
-	else if (quisk_record_state == PLAY_FILE)
+	else if (quisk_record_state == FILE_PLAY_SPKR_MIC)
 		quisk_file_playback(cSamples, nSamples, 1.0);		// replace radio sound
-   
 	// Play the demodulated audio
 #if DEBUG_MIC != 2
 	if ( ! quisk_play_sidetone(&quisk_Playback))	// play sidetone
@@ -972,16 +992,16 @@ int quisk_read_sound(void)	// Called from sound thread
 	else if (radio_sound_mic_socket != INVALID_SOCKET)
 		mic_count = read_radio_sound_socket(cSamples);
 	else {      // No mic source; use zero samples
-		mic_count = QuiskDeltaMsec(0) * mic_sample_rate / 1000;
+		mic_count = (int)(0.5 + QuiskDeltaSec(0) * mic_sample_rate);
 		if (mic_count > SAMP_BUFFER_SIZE / 2)
 			mic_count = SAMP_BUFFER_SIZE / 2;
 		for (i = 0; i < mic_count; i++)
 			cSamples[i] = 0;
 	}
 	//QuiskPrintTime("quisk_read_sound end", 0);
-	if (quisk_record_state == PLAYBACK)			// Discard previous samples and replace with saved sound
+	if (quisk_record_state == TMP_PLAY_SPKR_MIC)			// Discard previous samples and replace with saved sound
 		quisk_tmp_microphone(cSamples, mic_count);
-	else if (quisk_record_state == PLAY_FILE)	// Discard previous samples and replace with saved sound
+	else if (quisk_record_state == FILE_PLAY_SPKR_MIC)	// Discard previous samples and replace with saved sound
 		quisk_file_microphone(cSamples, mic_count);
 	if (DigitalInput.handle) {
 		if (is_DGT) {		// Discard previous mic samples and use digital samples
@@ -1004,6 +1024,13 @@ int quisk_read_sound(void)	// Called from sound thread
 	if (file_rec_mic.fp)
 		quisk_record_audio(&file_rec_mic, cSamples, mic_count);
 
+	// For remote_radio role in remote control operation, read mic sound via UDP; replace any mic sound from above.
+	if (remote_control_slave)
+		mic_count = read_remote_mic_sound_socket(&quisk_MicPlayback, cSamples);
+
+	// For control_head role in remote control operation, send mic sound via UDP
+	if (remote_control_head)
+		send_remote_mic_sound_socket(cSamples, mic_count);
 
 	if (mic_count > 0) {
 #if DEBUG_MIC == 3
@@ -1031,36 +1058,41 @@ int quisk_read_sound(void)	// Called from sound thread
 		}
 		else {
 			is_cw = 0;
+			cwCount = 0;
 			cwEnvelope = 0.0;
 		}
 		tx_mic_phase = cexp(( -I * 2.0 * M_PI * quisk_tx_tune_freq) / quisk_MicPlayback.sample_rate);
-		if (is_cw) {	// Transmit CW
+		if (is_cw) {	// Transmit CW; use capture device for timing, not microphone
+			cwCount += (double)retval * quisk_MicPlayback.sample_rate / quisk_sound_state.sample_rate;
+			mic_count = 0;
 			if (QUISK_CWKEY_DOWN || quiskSpotLevel >= 0) {
-				for (i = 0; i < mic_count; i++) {
+				while (cwCount >= 1.0) {
 					if (cwEnvelope < 1.0) {
 						cwEnvelope += 1. / (quisk_MicPlayback.sample_rate * 5e-3);	// 5 milliseconds
 						if (cwEnvelope > 1.0)
 							cwEnvelope = 1.0;
 					}
 					if (quiskSpotLevel >= 0)
-						cSamples[i] = (CLIP16 - 1) * cwEnvelope * quiskSpotLevel / 1000.0 * tuneVector * quisk_sound_state.mic_out_volume;
+						cSamples[mic_count++] = (CLIP16 - 1) * cwEnvelope * quiskSpotLevel / 1000.0 * tuneVector * quisk_sound_state.mic_out_volume;
 					else
-						cSamples[i] = (CLIP16 - 1) * cwEnvelope * tuneVector * quisk_sound_state.mic_out_volume;
+						cSamples[mic_count++] = (CLIP16 - 1) * cwEnvelope * tuneVector * quisk_sound_state.mic_out_volume;
 					tuneVector *= tx_mic_phase;
+					cwCount -= 1;
 				}
 			}
 			else {		// key is up
-				for (i = 0; i < mic_count; i++) {
+				while (cwCount >= 1.0) {
 					if (cwEnvelope > 0.0) {
 						cwEnvelope -= 1.0 / (quisk_MicPlayback.sample_rate * 5e-3);	// 5 milliseconds
 						if (cwEnvelope < 0.0)
 							cwEnvelope = 0.0;
 					}
 					if (quiskSpotLevel >= 0)
-						cSamples[i] = (CLIP16 - 1) * cwEnvelope * quiskSpotLevel / 1000.0 * tuneVector * quisk_sound_state.mic_out_volume;
+						cSamples[mic_count++] = (CLIP16 - 1) * cwEnvelope * quiskSpotLevel / 1000.0 * tuneVector * quisk_sound_state.mic_out_volume;
 					else
-						cSamples[i] = (CLIP16 - 1) * cwEnvelope * tuneVector * quisk_sound_state.mic_out_volume;
+						cSamples[mic_count++] = (CLIP16 - 1) * cwEnvelope * tuneVector * quisk_sound_state.mic_out_volume;
 					tuneVector *= tx_mic_phase;
+					cwCount -= 1;
 				}
 			}
 		}
@@ -1083,12 +1115,13 @@ int quisk_read_sound(void)	// Called from sound thread
 				tuneVector *= tx_mic_phase;
 			}
 		}
-		// delay the I or Q channel by one sample
-		if (quisk_MicPlayback.channel_Delay >= 0)
-			delay_sample(&quisk_MicPlayback, (double *)cSamples, mic_count);
+		// Correct order of delay and phase by Chuck Ritola.  Thanks Chuck.
 		// amplitude and phase corrections
 		if (quisk_MicPlayback.doAmplPhase)
 			correct_sample (&quisk_MicPlayback, cSamples, mic_count);
+		// delay the I or Q channel by one sample
+		if (quisk_MicPlayback.channel_Delay >= 0)
+			delay_sample(&quisk_MicPlayback, (double *)cSamples, mic_count);
 		// play mic samples
 		//quisk_sample_level("play quisk_MicPlayback", cSamples, mic_count, CLIP32);
 		play_sound_interface(&quisk_MicPlayback, mic_count, cSamples, 1, mic_play_volume);
@@ -1121,10 +1154,10 @@ void quisk_close_sound(void)	// Called from sound thread
 #ifdef MS_WINDOWS
 	int cleanup = radio_sound_socket != INVALID_SOCKET || radio_sound_mic_socket != INVALID_SOCKET;
 #endif
-	quisk_close_sound_directx(CaptureDevices, PlaybackDevices);
-	quisk_close_sound_wasapi(CaptureDevices, PlaybackDevices);
+	quisk_close_sound_directx(CaptureDevices, quiskPlaybackDevices);
+	quisk_close_sound_wasapi(CaptureDevices, quiskPlaybackDevices);
 	quisk_close_sound_portaudio();
-	quisk_close_sound_alsa(CaptureDevices, PlaybackDevices);
+	quisk_close_sound_alsa(CaptureDevices, quiskPlaybackDevices);
 	quisk_close_sound_pulseaudio();
 	if (pt_sample_stop)
 		(*pt_sample_stop)();
@@ -1247,25 +1280,52 @@ PyObject * quisk_set_sound_name(PyObject * self, PyObject * args)	// Called from
 	char * device_name;	// The device name from quisk.py
 	const char * utf8 = "utf-8";
 	int index, play, driver;
+	Py_ssize_t l1, l2;
 
-	if (!PyArg_ParseTuple (args, "iiieses", &index, &play, &driver, utf8, &description, utf8, &device_name))
+	description = malloc(QUISK_SC_SIZE);
+	device_name = malloc(QUISK_SC_SIZE);
+	l1 = l2 = QUISK_SC_SIZE;
+	if (!PyArg_ParseTuple (args, "iiies#es#", &index, &play, &driver, utf8, &description, &l1, utf8, &device_name, &l2))
 		return NULL;
 	if (play) {
-		strMcpy(PlaybackDevices[index]->name, description, QUISK_SC_SIZE);
-		strMcpy(PlaybackDevices[index]->device_name, device_name, QUISK_SC_SIZE);
-		PlaybackDevices[index]->driver = driver;
+		strMcpy(quiskPlaybackDevices[index]->name, description, QUISK_SC_SIZE);
+		strMcpy(quiskPlaybackDevices[index]->device_name, device_name, QUISK_SC_SIZE);
+		quiskPlaybackDevices[index]->driver = driver;
 	}
 	else {
 		strMcpy(CaptureDevices[index]->name, description, QUISK_SC_SIZE);
 		strMcpy(CaptureDevices[index]->device_name, device_name, QUISK_SC_SIZE);
 		CaptureDevices[index]->driver = driver;
 	}
-	PyMem_Free(description);
-	PyMem_Free(device_name);
+	free(description);
+	free(device_name);
 	Py_INCREF (Py_None);
 	return Py_None;
 }
 
+static void set_digital_rx()	// Set playback devices for digital modes from sub-receivers
+{
+	int i;
+	char buf80[80];
+	struct sound_dev * rx_dev;
+
+	for (i = QUISK_INDEX_SUB_RX1; i < QUISK_INDEX_SUB_RX1 + QUISK_MAX_SUB_RECEIVERS; i++) {
+		rx_dev = quiskPlaybackDevices[i];
+		rx_dev->dev_errmsg[0] = 0;
+		rx_dev->dev_index = t_DigitalRx1Output;
+		snprintf(buf80, 80, "Digital Rx%d Output", i - QUISK_INDEX_SUB_RX1 + 1);
+		strMcpy(rx_dev->stream_description, buf80, QUISK_SC_SIZE);
+		rx_dev->sample_rate = 48000;
+		rx_dev->channel_I = 0;
+		rx_dev->channel_Q = 1;
+		set_num_channels (rx_dev);
+		rx_dev->average_square = 0;
+		rx_dev->stream_dir_record = 0;
+		rx_dev->read_frames = 0;
+		rx_dev->latency_frames = rx_dev->sample_rate * quisk_sound_state.latency_millisecs / 1000;
+	}
+}
+
 void quisk_open_sound(void)	// Called from GUI thread
 {
 	int i;
@@ -1276,7 +1336,7 @@ void quisk_open_sound(void)	// Called from GUI thread
 	quisk_sound_state.write_error = 0;
 	quisk_sound_state.underrun_error = 0;
 	quisk_sound_state.mic_read_error = 0;
-	quisk_sound_state.interupts = 0;
+	quisk_sound_state.interrupts = 0;
 	quisk_sound_state.rate_min = quisk_sound_state.rate_max = -99;
 	quisk_sound_state.chan_min = quisk_sound_state.chan_max = -99;
 	quisk_sound_state.msg1[0] = 0;
@@ -1288,7 +1348,6 @@ void quisk_open_sound(void)	// Called from GUI thread
 	DigitalInput.dev_errmsg[0] = 0;
 	DigitalOutput.dev_errmsg[0] = 0;
 	RawSamplePlayback.dev_errmsg[0] = 0;
-	quisk_DigitalRx1Output.dev_errmsg[0] = 0;
 	Capture.dev_index = t_Capture;
 	quisk_Playback.dev_index = t_Playback;
 	MicCapture.dev_index = t_MicCapture;
@@ -1296,7 +1355,8 @@ void quisk_open_sound(void)	// Called from GUI thread
 	DigitalInput.dev_index = t_DigitalInput;
 	DigitalOutput.dev_index = t_DigitalOutput;
 	RawSamplePlayback.dev_index = t_RawSamplePlayback;
-	quisk_DigitalRx1Output.dev_index = t_DigitalRx1Output;
+
+	set_digital_rx();
 
 	// Set stream descriptions. This is important for "deviceless" drivers like
 	// PulseAudio to be able to distinguish the streams from each other.
@@ -1308,10 +1368,9 @@ void quisk_open_sound(void)	// Called from GUI thread
 	MicCapture.stream_description[QUISK_SC_SIZE-1] = '\0';
 	strMcpy(quisk_MicPlayback.stream_description, "I/Q Tx Sample Output", QUISK_SC_SIZE);
 	quisk_MicPlayback.stream_description[QUISK_SC_SIZE-1] = '\0';
-	strMcpy(DigitalInput.stream_description, "External Digital Input", QUISK_SC_SIZE);
-	strMcpy(DigitalOutput.stream_description, "External Digital Output", QUISK_SC_SIZE);
+	strMcpy(DigitalInput.stream_description, "Digital Tx0 Input", QUISK_SC_SIZE);
+	strMcpy(DigitalOutput.stream_description, "Digital Rx0 Output", QUISK_SC_SIZE);
 	strMcpy(RawSamplePlayback.stream_description, "Raw Digital Output", QUISK_SC_SIZE);
-	strMcpy(quisk_DigitalRx1Output.stream_description, "Digital Rx1 Output", QUISK_SC_SIZE);
    
 	quisk_Playback.sample_rate = quisk_sound_state.playback_rate;		// Radio sound play rate
 	quisk_MicPlayback.sample_rate = quisk_sound_state.mic_playback_rate;
@@ -1331,10 +1390,6 @@ void quisk_open_sound(void)	// Called from GUI thread
 	RawSamplePlayback.sample_rate = quisk_sound_state.sample_rate;
 	RawSamplePlayback.channel_I = 0;
 	RawSamplePlayback.channel_Q = 1;
-	// Playback device for digital modes from sub-receivers
-	quisk_DigitalRx1Output.sample_rate = 48000;
-	quisk_DigitalRx1Output.channel_I = 0;
-	quisk_DigitalRx1Output.channel_Q = 1;
 
 	set_num_channels (&Capture);
 	set_num_channels (&quisk_Playback);
@@ -1343,9 +1398,8 @@ void quisk_open_sound(void)	// Called from GUI thread
 	set_num_channels (&DigitalInput);
 	set_num_channels (&DigitalOutput);
 	set_num_channels (&RawSamplePlayback);
-	set_num_channels (&quisk_DigitalRx1Output);
 
-	for (i = 0; (pPlay = PlaybackDevices[i]); i++) {
+	for (i = 0; (pPlay = quiskPlaybackDevices[i]); i++) {
 		pPlay->started = 0;
 		pPlay->cr_correction = 0;
 		pPlay->cr_delay = 3;
@@ -1362,7 +1416,6 @@ void quisk_open_sound(void)	// Called from GUI thread
 	DigitalInput.average_square = 0;
 	DigitalOutput.average_square = 0;
 	RawSamplePlayback.average_square = 0;
-	quisk_DigitalRx1Output.average_square = 0;
 
 	//Needed for pulse audio context connection (KM4DSJ)
 	Capture.stream_dir_record = 1;
@@ -1372,7 +1425,6 @@ void quisk_open_sound(void)	// Called from GUI thread
 	DigitalInput.stream_dir_record = 1;
 	DigitalOutput.stream_dir_record = 0;
 	RawSamplePlayback.stream_dir_record = 0;
-	quisk_DigitalRx1Output.stream_dir_record = 0;
 
 	//For remote IQ server over pulseaudio (KM4DSJ)
 	if (quisk_sound_state.IQ_server[0]) {
@@ -1410,8 +1462,6 @@ void quisk_open_sound(void)	// Called from GUI thread
 	DigitalInput.latency_frames = 0;
 	DigitalOutput.read_frames = 0;
 	DigitalOutput.latency_frames = DigitalOutput.sample_rate * quisk_sound_state.latency_millisecs / 1000;
-	quisk_DigitalRx1Output.read_frames = 0;
-	quisk_DigitalRx1Output.latency_frames = quisk_DigitalRx1Output.sample_rate * quisk_sound_state.latency_millisecs / 1000;
 	// set capture and playback for raw samples
 	RawSamplePlayback.read_frames = 0;
 	RawSamplePlayback.latency_frames = RawSamplePlayback.sample_rate * quisk_sound_state.latency_millisecs / 1000;
@@ -1427,11 +1477,11 @@ void quisk_start_sound(void)	// Called from sound thread
 		(*pt_sample_start)();
    
 	// Let the drivers see the devices and start them up if appropriate
-	quisk_start_sound_directx(CaptureDevices, PlaybackDevices);
-	quisk_start_sound_wasapi(CaptureDevices, PlaybackDevices);
-	quisk_start_sound_portaudio(CaptureDevices, PlaybackDevices);
-	quisk_start_sound_pulseaudio(CaptureDevices, PlaybackDevices);
-	quisk_start_sound_alsa(CaptureDevices, PlaybackDevices);
+	quisk_start_sound_directx(CaptureDevices, quiskPlaybackDevices);
+	quisk_start_sound_wasapi(CaptureDevices, quiskPlaybackDevices);
+	quisk_start_sound_portaudio(CaptureDevices, quiskPlaybackDevices);
+	quisk_start_sound_pulseaudio(CaptureDevices, quiskPlaybackDevices);
+	quisk_start_sound_alsa(CaptureDevices, quiskPlaybackDevices);
 	if (pt_sample_read) {	// Capture from SDR-IQ or UDP
 		quisk_sound_state.rate_min = quisk_Playback.rate_min;
 		quisk_sound_state.rate_max = quisk_Playback.rate_max;
@@ -1444,8 +1494,8 @@ void quisk_start_sound(void)	// Called from sound thread
 		quisk_sound_state.chan_min = Capture.chan_min;
 		quisk_sound_state.chan_max = Capture.chan_max;
 	}
-	QuiskDeltaMsec(0);	// Set timer to zero
-	QuiskDeltaMsec(1);
+	QuiskDeltaSec(0);	// Set timer to zero
+	QuiskDeltaSec(1);
 	quisk_set_play_state();
 	quisk_play_state = STARTING;
 }
@@ -1504,9 +1554,11 @@ PyObject * quisk_micplay_channels(PyObject * self, PyObject * args)	// Called fr
 
 PyObject * quisk_set_sparams(PyObject * self, PyObject * args, PyObject * keywds)
 {  /* Call with keyword arguments ONLY; change local parameters */
-	static char * kwlist[] = {"dc_remove_bw", "digital_output_level", NULL} ;
+	static char * kwlist[] = {"dc_remove_bw", "digital_output_level", "remote_control_slave", "remote_control_head",
+		NULL} ;
 
-	if (!PyArg_ParseTupleAndKeywords (args, keywds, "|id", kwlist, &dc_remove_bw, &digital_output_level))
+	if (!PyArg_ParseTupleAndKeywords (args, keywds, "|idii", kwlist, &dc_remove_bw, &digital_output_level,
+	&remote_control_slave, &remote_control_head))
 		return NULL;
 	Py_INCREF (Py_None);
 	return Py_None;
@@ -1551,23 +1603,25 @@ static void AddCard(struct sound_dev * dev, PyObject * pylist)
 PyObject * quisk_sound_errors(PyObject * self, PyObject * args)
 {  // return a list of strings with card names and error counts
 	PyObject * pylist;
+	struct sound_dev * rx_dev;
 
 	if (!PyArg_ParseTuple (args, ""))
 		return NULL;
 	pylist = PyList_New(0);
 	AddCard(&Capture,	pylist);
 	AddCard(&MicCapture,	pylist);
-	AddCard(&DigitalInput,	pylist);
 	AddCard(&quisk_Playback,	pylist);
 	AddCard(&quisk_MicPlayback,	pylist);
-	AddCard(&DigitalOutput,	pylist);
 	AddCard(&RawSamplePlayback, pylist);
-	AddCard(&quisk_DigitalRx1Output, pylist);
+	AddCard(&DigitalInput,	pylist);
+	AddCard(&DigitalOutput,	pylist);
+	rx_dev = quiskPlaybackDevices[QUISK_INDEX_SUB_RX1];
+	AddCard(rx_dev, pylist);
 	return pylist;
 }
 
 PyObject * quisk_set_file_name(PyObject * self, PyObject * args, PyObject * keywds)  // called from GUI
-{  // Set the names and enable state of the recording and playback files.
+{
 	int which = -1;
 	const char * name = NULL;
 	int enable = -1;
@@ -1577,54 +1631,34 @@ PyObject * quisk_set_file_name(PyObject * self, PyObject * args, PyObject * keyw
 
 	if (!PyArg_ParseTupleAndKeywords (args, keywds, "|isiii", kwlist, &which, &name, &enable, &play_button, &record_button))
 		return NULL;
-	switch (which) {
-	case 0:		// record audio file
-		if (name)
-			strMcpy(file_rec_audio.file_name, name, QUISK_PATH_SIZE);
-		if (enable != -1)
-			file_rec_audio.enable = enable;
-		break;
-	case 1:		// record sample file
-		if (name)
-			strMcpy(file_rec_samples.file_name, name, QUISK_PATH_SIZE);
-		if (enable != -1)
-			file_rec_samples.enable = enable;
-		break;
-	case 2:		// record mic file
-		if (name)
-			strMcpy(file_rec_mic.file_name, name, QUISK_PATH_SIZE);
-		if (enable != -1)
-			file_rec_mic.enable = enable;
-		break;
-	case 10:	// play audio file
-		break;
-	case 11:	// play samples file
-		break;
-	case 12:	// play CQ message file
-		break;
+	if (record_button == 0) {	// Close all recording files
+		close_file_rec = 1;
 	}
-	if (record_button != -1)
-		file_record_button = record_button;
-	if (file_rec_audio.enable && file_record_button){	// Open and Close Rx audio file
-		if ( ! file_rec_audio.fp)
-			quisk_record_audio(&file_rec_audio, NULL, -1);	// Open file
+	if (play_button == 0) {	// Close all play files
+		quisk_close_file_play = 1;
 	}
-	else if (file_rec_audio.fp) {
-		quisk_record_audio(&file_rec_audio, NULL, -2);		// Close file
-	}
-	if (file_rec_mic.enable && file_record_button){		// Open and Close microphone audio file
-		if ( ! file_rec_mic.fp)
-			quisk_record_audio(&file_rec_mic, NULL, -1);
-	}
-	else if (file_rec_mic.fp) {
-		quisk_record_audio(&file_rec_mic, NULL, -2);
-	}
-	if (file_rec_samples.enable && file_record_button){ // Open and Close I/Q samples file
-		if ( ! file_rec_samples.fp)
+	if (record_button == 1) switch (which) {
+		case 0:		// record audio file
+			if (name)
+				strMcpy(file_rec_audio.file_name, name, QUISK_PATH_SIZE);
+			quisk_record_audio(&file_rec_audio, NULL, -1);
+			break;
+		case 1:		// record sample file
+			if (name)
+				strMcpy(file_rec_samples.file_name, name, QUISK_PATH_SIZE);
 			record_samples(&file_rec_samples, NULL, -1);
-	}
-	else if (file_rec_samples.fp) {
-		record_samples(&file_rec_samples, NULL, -2);
+			break;
+		case 2:		// record mic file
+			if (name)
+				strMcpy(file_rec_mic.file_name, name, QUISK_PATH_SIZE);
+			quisk_record_audio(&file_rec_mic, NULL, -1);
+			break;
+		case 10:	// play audio file
+			break;
+		case 11:	// play samples file
+			break;
+		case 12:	// play CQ message file
+			break;
 	}
 	Py_INCREF (Py_None);
 	return Py_None;
