@@ -1160,7 +1160,7 @@ PyObject * quisk_alsa_control_midi(PyObject * self, PyObject * args, PyObject *
 	if (device) {		// open port
 		state = 0;
 		quisk_midi_cwkey = 0;
-		if (snd_rawmidi_open(&handle_in, NULL, device, 0) != 0) {
+		if (snd_rawmidi_open(&handle_in, NULL, device, SND_RAWMIDI_NONBLOCK) != 0) {
 			handle_in = NULL;
 			printf("Failed to open MIDI device %s\n", device);
 		}
